package org.expeditee.actions;

import org.expeditee.gui.Browser;
import org.expeditee.gui.DisplayIO;
import org.expeditee.gui.Frame;
import org.expeditee.gui.FrameGraphics;
import org.expeditee.gui.FrameIO;
import org.expeditee.gui.FrameMouseActions;
import org.expeditee.gui.FrameUtils;
import org.expeditee.gui.FreeItems;
import org.expeditee.gui.MessageBay;
import org.expeditee.io.WebParser;
import org.expeditee.items.Item;
import org.expeditee.items.Text;
import org.expeditee.items.widgets.InteractiveWidget;
import org.expeditee.items.widgets.InteractiveWidgetInitialisationFailedException;
import org.expeditee.items.widgets.InteractiveWidgetNotAvailableException;
import org.expeditee.items.widgets.JfxBrowser;
import org.expeditee.settings.network.NetworkSettings;



/**
 * A list of Actions used with the JFX Browser widget
 * 
 */
public class JfxBrowserActions {
	
	/**
	 * Launches items.widgets.JfxBrowser and uses Text item as URL.
	 * @param text Text item which passes contents as URL for browser.
	 * @throws Exception
	 */
	public static void startBrowser(Item text) throws Exception {
		if (!(text instanceof Text)) {
			MessageBay.errorMessage("Must be a text item.");
			return;
		}
		if(text.getLink() != null) {
			MessageBay.errorMessage("Text item cannot have link.");
			return;
		}
			
		Text wt = new Text("@iw:org.expeditee.items.widgets.JfxBrowser");	// create new text item for browser widget
		
		if(FreeItems.textOnlyAttachedToCursor()) {			// navigates to url specified by the text item
			wt.appendText(":" + text.getText());
		} else {
			wt.appendText(":http://www.waikato.ac.nz");
		}
		
		FreeItems.getInstance().clear();					// remove url text from cursor
		
		wt.setParent(DisplayIO.getCurrentFrame());			// set parent of text source for InteractiveWidget.createWidget()
		wt.setXY(FrameMouseActions.getX(), FrameMouseActions.getY());
		
		// create widget from text item
		JfxBrowser browser = (JfxBrowser) InteractiveWidget.createWidget(wt);
			
		FrameMouseActions.pickup(browser.getItems());		// attach browser widget to mouse
	}
	
	/**
	 * Text item becomes link to new frame containing items.widgets.JfxBrowser and uses Text item as URL for browser.
	 * @param text Text item which passes contents as URL for browser and becomes link to the browser's new frame.
	 * @throws Exception
	 */
	public static void startBrowserNewFrame(Item text) throws Exception {
		if (!(text instanceof Text)) {
			MessageBay.errorMessage("Must be a text item.");
			return;
		}
		if(text.getLink() != null) {						// text item can't already have a link
			MessageBay.errorMessage("Text item already has link.");
			return;
		}
		
		// If no text with url is passed to action create a new text item with http://www.waikato.ac.nz for a default url
		if(!FreeItems.textOnlyAttachedToCursor()) {
			text = DisplayIO.getCurrentFrame().addText(FrameMouseActions.getX(), FrameMouseActions.getY(), 
					NetworkSettings.HomePage.get(), null);
			text.setParent(DisplayIO.getCurrentFrame());	// set parent of text source for InteractiveWidget.createWidget()
			FrameMouseActions.pickup(text);					// Attach new text link to cursor
		}
		
		// Create JfxBrowser widget on a new frame
		Frame frame = FrameIO.CreateNewFrame(text);			// create new frame for browser
		frame.removeAllItems(frame.getItems());
		text.setLink("" + frame.getNumber());				// link this text item to new frame
		
		// Create widget via text annotation
		Text wt = frame.addText(0, 0, "@iw: org.expeditee.items.widgets.JfxBrowser " 
				+ Browser._theBrowser.getContentPane().getWidth() +  " " + Browser._theBrowser.getContentPane().getHeight() 
				+ " : " + text.getText(), null);
		
		InteractiveWidget.createWidget(wt);
		
		FrameIO.SaveFrame(frame);							// save frame to disk
	}
	
	public static void parsePage(Item text) throws Exception {
		if (!(text instanceof Text) || !FreeItems.textOnlyAttachedToCursor()) {
			MessageBay.errorMessage("Must provide a text item.");
			return;
		}
		if(text.getLink() != null) {						// text item can't already have a link
			MessageBay.errorMessage("Text item already has link.");
			return;
		}
		
		// Create JfxBrowser widget on a new frame
		Frame frame = FrameIO.CreateNewFrame(text);			// create new frame for browser
		text.setLink("" + frame.getNumber());				// link this text item to new frame
		frame.addText(100, 100, "test", null);
		
		WebParser.parseURL(text.getText(), frame);
		System.out.println(text.getText());
	}
	
	public static void gotoURL(Text link, String URL) {
		Frame frame = FrameIO.CreateNewFrame(link);
		link.setAction(null);
		link.setLink("" + frame.getNumber());
		
		WebParser.parseURL(URL, frame);
		
		frame.change();
		
		FrameIO.SaveFrame(frame);
	}

	/**
	 * Creates a frame containing a JFXBrowser pointed to the specified URL, then navigates to that frame
	 * 
	 * @param link
	 *            Item that will link to the new frame
	 * @param url
	 *            URL to load in the Web Browser of the new frame
	 * @throws Exception
	 */
	public static void createFrameWithBrowser(Text link, String url) throws Exception {
		Frame frame = FrameIO.CreateNewFrame(link);
		frame.removeAllItems(frame.getItems());
		link.setLink("" + frame.getNumber());
		link.setAction(null);

		// Create widget via text annotation
		Text wt = frame.addText(10, 10, "@iw: org.expeditee.items.widgets.JfxBrowser " + (int) (FrameGraphics.getMaxFrameSize().getWidth() * 0.9) + " "
				+ (int) (FrameGraphics.getMaxFrameSize().getHeight() * 0.9) + " : " + url, null);

		InteractiveWidget.createWidget(wt);

		FrameIO.SaveFrame(frame);

		FrameUtils.DisplayFrame(link.getAbsoluteLink());
	}
}
